<?php defined('SYSPATH') || die('No direct script access.');

/**
 * Class Model_RecordBook
 *
 * @property-read   $ID             int
 * @property        $ExternalID     string
 * @property        $PlanID         int
 * @property        $SubdivisionID  int
 * @property        $Status         string
 * @property        $GroupID        int
 * @property        $GroupNum       int
 * @property        $GroupName      string
 * @property        $GradeID        int
 * @property        $GradeNum       int
 * @property        $Degree         string
 * @property        $SpecID         int
 * @property        $SpecName       string
 * @property        $SpecAbbr       string
 * @property        $SpecCode       string
 * @property        $FacultyID      int
 * @property        $FacultyName    string
 * @property        $FacultyAbbr    string
 */
class Model_RecordBook extends Model_Container
{
    protected function getRawData($id) {
        $sql = 'SELECT * FROM RecordBook_GetInfo(:id)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $id)->execute();

        if ($info->count() == 0)
            throw new InvalidArgumentException('RecordBook not found');

        return $info[0];
    }

    public static function make() {
        return new Model_Helper_RecordBookBuilder();
    }

    protected function create() {
        $sql = 'SELECT * FROM RecordBook_Bind(ExternalID, StudentID, PlanID) AS "ID"';
        $this->data[self::$ID_FIELD] = DB::query(Database::SELECT, $sql)
            ->parameters($this->data)
            ->execute()->get('ID');

        if ($this->ID <= 0)
            throw new InvalidArgumentException(Error::INVALID_PARAMETERS);

        if (isset($this->data['GroupID'])) {
            $this->transferIntoGroup($this->data['GroupID'], Model_System::$SemesterID);
        }
    }

    public static function withExternalID($extid) {
        $sql = 'SELECT * FROM RecordBook_GetIDFromExternalID(:ExtID) AS "ID"';
        $res = DB::query(Database::SELECT, $sql)
            ->param(':ExtID', $extid)
            ->execute()->get('ID');
        return (int)$res;
    }

    /** @return Model_Discipline[] */
    public function getDisciplines($semesterID = null) {
        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $recordBookID = $this->ID;
        
        $sql = 'SELECT * FROM Student_GetDisciplines(:recordBookID, :semesterID)';
        $query = DB::query(Database::SELECT, $sql)
            ->param(':recordBookID', $recordBookID)
            ->param(':semesterID', $semesterID)
            ->execute();

        $list = [];
        foreach ($query as $data)
            $list[] = new Model_Discipline($data, true);

        return $list;
    }

    public function getTeachers($loadAll = false, $semesterID = null) {
        // todo: don't load the full data at the Controller_Student_Index
        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $recordBookID = $this->ID;

        $sql = 'SELECT * FROM Student_GetTeachersList(:recordBookID, :semesterID, :loadAll)';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':recordBookID', $recordBookID)
            ->param(':semesterID', $semesterID)
            ->param(':loadAll', $loadAll)
            ->execute();

        $list = [];
        foreach ($result as $row) {
            $id = $row['DisciplineID'];
            $list[$id][] = $row;
        }
        return $list;
    }

    public function update($data) {
        $updatedData = array_merge(['ExternalID' => null], $this->data, $data);
        $sql = 'SELECT * FROM RecordBook_ChangeInfo(ID, ExternalID, StudentID, PlanID) AS "ErrorCode"';

        $res = (int)DB::query(Database::SELECT, $sql)
            ->parameters($updatedData)
            ->execute()->get('"ErrorCode"');
        if ($res < 0) {
            throw new Database_Exception('RecordBook update failed!');
        }

        if (isset($updatedData['GroupID'])) {
            $this->transferIntoGroup($updatedData['GroupID'], Model_System::$SemesterID);
        }
    }

    /**
     * Set the student's state and group in a semester.
     * @param $group int  group id
     * @return $this;
     */
    public function setState($group, $state="common", $semesterID = null) {
        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $sql = 'SELECT * FROM ControlStudentGroup(:id, :group, :state, :semesterID) AS "Result"';

        DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':group', (int) $group)
            ->param(':state', $state)
            ->param(':semesterID', $semesterID)
            ->execute();

        return $this;
    }

    /**
     * Bind student to a global discipline
     * @param $discipline int  discipoline id
     * @return $this;
     */
    public static function bindInternal($recordBook, $discipline) {
        $sql = 'SELECT * FROM InternalBindStudent(:disciplineID, :id) AS "Result"';

        $res = DB::query(Database::SELECT, $sql)
            ->param(':disciplineID', $discipline)
            ->param(':id', $recordBook)
            ->execute()->get("Result");

        return $res;
    }

    /**
     * Find recordbook by student nickname on remote service
     * and partial subject name, semester id
     * @param $discipline int  discipoline id
     * @return $this;
     */
    public static function getByNickSubject($nick, $subject, $semester, $service) {
        $sql = 'SELECT * FROM GetrecordbookForSubjectAndNick(:nick, :subject, :semester, :service) AS "Result"';

        $res = DB::query(Database::SELECT, $sql)
            ->param(':nick', $nick)
            ->param(':subject', $subject)
            ->param(':semester', $semester)
            ->param(':service', $service)
            ->execute()->as_array();

        return $res;
    }

    public function transferIntoGroup($group, $semesterID = null) {
        return $this->setState($group, "common", $semesterID);
    }
}
